@@ -3,7 +3,7 @@
 # Authors: Arthur Michaut                                                #
 # Copyright 2016-2019 Harvard Medical School and Brigham and             #
 #                          Women's Hospital                              #
-# Copyright 2019-2021 Institut Pasteur and CNRS–UMR3738                  #
+# Copyright 2019-2022 Institut Pasteur and CNRS–UMR3738                  #
 # See the COPYRIGHT file for details                                     #
 #                                                                        #
 # This file is part of Track Analyzer package.                           #
@@ -37,6 +37,8 @@ from scipy.optimize import curve_fit
 
 from track_analyzer import prepare as tpr
 
+pd.options.mode.chained_assignment = None  # remove SettingWithCopyWarning
+
 
 def regularize_traj(df, dimensions=['x', 'y', 'z'], split_traj=False):
     """This function either splits traj with time gaps, or fill the gap by linear interpolation (default).
@@ -166,35 +168,51 @@ def recompute_pos(df, dimensions=['x', 'y', 'z'], lengthscale=1.):
         df[dim] = df[dim + '_scaled'] * lengthscale
 
 
-def compute_track_prop(df, dimensions=['x', 'y', 'z']):
+def compute_track_prop(df, dimensions=['x', 'y', 'z'],custom_var={}):
     """Compute track properties by averaging all data along a traj"""
 
-    groups = df.groupby('track')
-
     # colmuns to average
     scaled_dimensions = [dim + '_scaled' for dim in dimensions]
     vel = ['v' + dim for dim in dimensions]
     acc = ['a' + dim for dim in dimensions]
-    mean_col = ['t'] + dimensions + scaled_dimensions + vel + acc + ['v', 'a']
+    col_to_avg = ['t'] + dimensions + scaled_dimensions + vel + acc + ['v', 'a'] + list(custom_var.keys())
     
     # all columns
-    columns = ['track', 'track_length'] + mean_col
+    columns = ['track', 'track_length'] + col_to_avg
     if 'subset' in df.columns:
         columns.append('subset')
-    df_out = pd.DataFrame(columns=columns)
+    df_mean = pd.DataFrame(columns=columns)
+    df_std = pd.DataFrame(columns=columns)
+
+    #check custom_var are numeric
+    for var in custom_var.keys():
+        df[var] = pd.to_numeric(df[var],errors='coerce')
 
     # average
-    for i, track in enumerate(df['track'].unique()):
-        traj = groups.get_group(track)
-        track_length = traj['t'].max() - traj['t'].min()
-        val = [track, track_length]
-        for col in mean_col:
-            val.append(traj[col].mean())
-        if 'subset' in df.columns:
-            val.append(traj['subset'].values[0])
-        df_out.loc[i, columns] = val
+    groups, group_values = tpr.get_unique_track_groups(df)
 
-    return df_out
+    for i, group_value in enumerate(group_values):
+        traj = groups.get_group(group_value)
+        track_length = traj['t'].max() - traj['t'].min()
+        if type(group_value) is tuple:
+            subset = group_value[0] 
+            track = group_value[1] 
+        else: 
+            track = group_value
+
+        mean_val = [track,track_length]
+        std_val = [track,track_length]
+        for col in col_to_avg:
+            mean_val.append(traj[col].mean())
+        for col in col_to_avg:
+            std_val.append(traj[col].std())
+        if 'subset' in df.columns:
+            mean_val.append(subset)
+            std_val.append(subset)
+        df_mean.loc[i,:] = mean_val
+        df_std.loc[i,:] = std_val
+
+    return df_mean, df_std
 
 
 def interpolate_field(data_dir, df, groups, grids, frame, field_values=['vx', 'vy', 'vz', 'ax', 'ay', 'az', 'v', 'a'],
@@ -246,14 +264,25 @@ def interpolate_field(data_dir, df, groups, grids, frame, field_values=['vx', 'v
         if frame_ in df['frame'].unique():
             group_ = groups.get_group(frame_).reset_index(drop=True)
             no_nan_ = group_[np.isfinite(group_[coord_list])]
-            triang = tri.Triangulation(no_nan_['x'].values, no_nan_['y'].values)
-            for coord in field_values:
-                interpolator = tri.LinearTriInterpolator(triang, no_nan_[coord].values)
-                field_dict[coord].append(interpolator(X, Y))
+
+            if no_nan_.shape[0] > 2:  # cannot triangulate with less then 3 points
+                triang = tri.Triangulation(no_nan_['x'].values, no_nan_['y'].values)
+                for coord in field_values:
+                    interpolator = tri.LinearTriInterpolator(triang, no_nan_[coord].values)
+                    field_dict[coord].append(interpolator(X, Y))
+            else:  # make fully masked array 
+                mask = np.ones(X.shape, dtype=bool)
+                marray = np.ma.ones(X.shape)
+                marray[:] = np.nan
+                marray.mask = mask
+                for coord in field_values:
+                    field_dict[coord].append(marray)
+
     for coord in field_values:
         # average interpolated vfield over the frame list
-        stack = np.ma.stack(field_dict[coord], axis=2)
-        field = np.ma.filled(np.sum(stack, axis=2) / len(frame_l), np.nan)
+        stack = np.ma.stack(field_dict[coord], axis=2)  # stack frames along axis 2
+        denom = len(frame_l) - np.sum(stack.mask, axis=2)  # denominator = number of non-masked data for each frame
+        field = np.ma.filled(np.sum(stack, axis=2) / denom, np.nan)
         data[coord] = field
 
     if export_field:
@@ -590,10 +619,10 @@ def compute_all_Voronoi(data_dir, df, outdir=None, compute_local_area=True, area
     # calculate track average
     if df_mean is not None:
         df_mean['area'] = np.nan
-        groups = df.groupby('track')
-        for track in df['track'].unique():
-            traj = groups.get_group(track)
-            ind = df_mean['track'] == track
+        track_groups, track_groups_values = tpr.get_unique_track_groups(df)
+        for tgv in track_groups_values:
+            traj = tpr.get_traj(track_groups, tgv)
+            ind = ((df_mean['subset'] == tgv[0]) & (df_mean['track'] == tgv[1])) if type(tgv) is tuple else df_mean['track'] == tgv
             df_mean.loc[ind, 'area'] = traj['area'].mean()
 
     return data_dict
@@ -847,7 +876,8 @@ def fit_msd(msd, mean_vel=None, traj=None, dim=2, model_bounds={'P': [0, 300], '
             raise Exception("Error: mean_vel is required to fit with {}. Aborting...".format(model))
         if model == 'PRW':
             param_list = ['P']
-            func = lambda t, P: 2 * (mean_vel ** 2) * P * (t - P * (1 - np.exp(-t / P)))
+            # mistake in Liu et al 2015, use Stokes 1991 definition instead
+            func = lambda t, P: (mean_vel ** 2) * P * (t - P * (1 - np.exp(-t / P)))  
             func_model = Model(func)
             p = func_model.make_params(P=10)
             p['P'].set(min=model_bounds['P'][0], max=model_bounds['P'][1])
