@@ -3,7 +3,7 @@
 # Authors: Arthur Michaut                                                #
 # Copyright 2016-2019 Harvard Medical School and Brigham and             #
 #                          Women's Hospital                              #
-# Copyright 2019-2021 Institut Pasteur and CNRS–UMR3738                  #
+# Copyright 2019-2022 Institut Pasteur and CNRS–UMR3738                  #
 # See the COPYRIGHT file for details                                     #
 #                                                                        #
 # This file is part of Track Analyzer package.                           #
@@ -42,6 +42,8 @@ import tifffile as tifff
 from track_analyzer import plotting as tpl
 from track_analyzer import calculate as tca
 
+pd.options.mode.chained_assignment = None  # remove SettingWithCopyWarning
+
 # Plotting parameters
 color_list = [c['color'] for c in list(plt.rcParams['axes.prop_cycle'])] + sns.color_palette("Set1", n_colors=9,
                                                                                              desat=.5)
@@ -63,7 +65,7 @@ def get_cmap_color(value, colormap='plasma', vmin=None, vmax=None):
     """
     Get color corresponding to a value from a colormap. Optionally, give boundaries to colormap with vmin, vmax.
     :param value: value to be converted to color
-    :type value: float
+    :type value: float or list or numpy.array
     :param colormap: Matplotlib colormap name
     :type colormap: str
     :param vmin: if not None, minimum value of colormap
@@ -73,11 +75,39 @@ def get_cmap_color(value, colormap='plasma', vmin=None, vmax=None):
     :return: color
     :rtype: tuple
     """
+
     colormap = plt.get_cmap(colormap)
     norm = plt.Normalize(vmin, vmax)
     return colormap(norm(value))
 
 
+def get_cmap_lim(df,color_code,cmap_lim,dim):
+    """
+    Get cmap limits depending on color_code special cases and make sanity checks
+    """
+
+    # special cases
+    if color_code in ['group','random','none']: 
+        return None, color_code
+    else:
+        if color_code == 'z': 
+            if dim != 3: 
+                print("WARNING: z color-code cannot be used with 2D data, changing to random")
+                return None, 'random'
+        cmap_lim_default = [df[color_code].min(), df[color_code].max()]
+
+    # use cmap_default value
+    cmap_lim = cmap_lim_default
+    # if cmap_lim is given in appropriate format (list or tuple of 2 elements, use is if not None)
+    if type(cmap_lim) is list or type(cmap_lim) is tuple:
+        if len(cmap_lim) == 2:
+            for i,lim in enumerate(cmap_lim):
+                if lim is None: 
+                    cmap_lim[i] = cmap_lim_default[i]
+
+    return cmap_lim, color_code
+
+
 def listdir_nohidden(path):
     """
     List a directory without hidden files starting with a dot
@@ -141,7 +171,7 @@ def make_unit_label(dimension='L', l_unit='um', t_unit='min'):
     :return: label as a Latex formatted string
     :rtype: str
     """
-    l_unit_dict = {'um': r'\mu m', 'mm': r'mm', 'px': 'px', 'none': '', 'au': ''}
+    l_unit_dict = {'um': r'\mu m', 'mm': r'mm', 'm': r'm', 'px': 'px', 'none': '', 'au': ''}
     t_unit_dict = {'min': r'min', 's': r's', 'frame': 'frame', 'none': '', 'au': ''}
 
     if dimension == 'L':
@@ -171,7 +201,7 @@ def make_unit_label(dimension='L', l_unit='um', t_unit='min'):
     return label
 
 
-def make_param_label(param, l_unit='um', t_unit='min', time_der=None, mean=False, only_symbol=False, only_unit=False):
+def make_param_label(param, l_unit='um', t_unit='min', time_der=None, mean=False, only_symbol=False, only_unit=False, manual_symbol=None, manual_unit=None):
     """
     Make a Latex formatted label of a parameter.
     The first and second time derivative and the mean symbol can be used.
@@ -193,6 +223,10 @@ def make_param_label(param, l_unit='um', t_unit='min', time_der=None, mean=False
     :type only_symbol: bool
     :param only_unit: return only the parameter unit
     :type only_unit: bool
+    :param manual_symbol: custom symbol to be used with param=None
+    :type manual_symbol: str or None
+    :param manual_unit: custom unit to be used with param=None
+    :type manual_unit: str or None
     :return: parameter label: symbol and/or unit
     :rtype: str
     """
@@ -220,61 +254,90 @@ def make_param_label(param, l_unit='um', t_unit='min', time_der=None, mean=False
     param_dict['track'] = {'sym': 'track id', 'dim': 'none', 'l_unit': l_unit, 't_unit': t_unit, 'latex': False}
     param_dict['area'] = {'sym': 'area', 'dim': 'LL', 'l_unit': l_unit, 't_unit': t_unit, 'latex': True}
 
-    # check if mean
-    if param.endswith('_mean'):
-        param = param[:param.find('_mean')]
-        mean = True
-
-    # convert param if it ends by _dot or _ddot
-    raw_param, time_der_ = get_param_time_der(param)
-    if time_der_ is not None:
-        time_der = time_der_
-        param = raw_param
-
-    # make symbol adding derivative or mean symbols
-    symbol = param_dict[param]['sym']
-    dim_der = ''
-    if time_der == 'first':
-        symbol = r'\dot{' + symbol + r'}'
-        dim_der = '/T'  # change dimension
-    elif time_der == 'sec':
-        symbol = r'\ddot{' + symbol + r'}'
-        dim_der = '/TT'  # change dimension
-    if mean:
-        symbol = r'\langle ' + symbol + r' \rangle'
-
-    # make unit
-    unit_ = make_unit_label(param_dict[param]['dim'] + dim_der, l_unit=param_dict[param]['l_unit'],
-                            t_unit=param_dict[param]['t_unit'])
-
-    # output
+    # output format
     if only_symbol and only_unit:
         print("Warning: only_symbol and only_unit can't be both True. Making them both False.")
         only_symbol = False
         only_unit = False
 
-    label = ''
-    # add latex dollar symbols if needed
-    if param_dict[param]['latex']:
-        if only_symbol:
-            label = r'$' + symbol + r'$'
-        elif only_unit:
-            label = r'$' + unit_ + r'$'
-        else:
-            if len(unit_) > 0:  # if unit exists
-                label = r'$' + symbol + r'$ ($' + unit_ + r'$)'
-            else:
+    # output label
+    label = ''  
+    if param is not None: 
+        # check if mean
+        if param.endswith('_mean'):
+            param = param[:param.find('_mean')]
+            mean = True
+
+        # convert param if it ends by _dot or _ddot
+        raw_param, time_der_ = get_param_time_der(param)
+        if time_der_ is not None:
+            time_der = time_der_
+            param = raw_param
+
+        # make symbol adding derivative or mean symbols
+        symbol = param_dict[param]['sym']
+        dim_der = ''
+        if time_der == 'first':
+            symbol = r'\dot{' + symbol + r'}'
+            dim_der = '/T'  # change dimension
+        elif time_der == 'sec':
+            symbol = r'\ddot{' + symbol + r'}'
+            dim_der = '/TT'  # change dimension
+        if mean:
+            symbol = r'\langle ' + symbol + r' \rangle'
+
+        # make unit
+        unit_ = make_unit_label(param_dict[param]['dim'] + dim_der, l_unit=param_dict[param]['l_unit'],
+                                t_unit=param_dict[param]['t_unit'])
+
+        # add latex dollar symbols if needed
+        if param_dict[param]['latex']:
+            if only_symbol:
                 label = r'$' + symbol + r'$'
-    else:
-        if only_symbol:
-            label = symbol
-        elif only_unit:
-            label = unit_
-        else:
-            if len(unit_) > 0:  # if unit exists
-                label = symbol + ' (' + unit_ + ')'
+            elif only_unit:
+                label = r'$' + unit_ + r'$'
             else:
+                if len(unit_) > 0:  # if unit exists
+                    label = r'$' + symbol + r'$ ($' + unit_ + r'$)'
+                else:
+                    label = r'$' + symbol + r'$'
+        else:
+            if only_symbol:
                 label = symbol
+            elif only_unit:
+                label = unit_
+            else:
+                if len(unit_) > 0:  # if unit exists
+                    label = symbol + ' (' + unit_ + ')'
+                else:
+                    label = symbol
+    else: 
+        no_symbol = False
+        no_unit = False
+        if manual_unit is None:
+            no_unit = True 
+        else:
+            if len(manual_unit) == 0:
+                no_unit = True
+        if manual_symbol is None:
+            no_symbol = True 
+        else:
+            if len(manual_symbol) == 0:
+                no_symbol = True
+
+        if only_symbol and not no_symbol:
+            label = r'' + manual_symbol
+        elif only_unit and not no_unit:
+            label = r'' + manual_unit
+        elif not only_symbol and not only_unit:
+            if no_symbol and not no_unit:
+                label = r'' + manual_unit
+            elif not no_symbol and no_unit:
+                label = r'' + manual_symbol
+            elif not no_symbol and not no_unit: 
+                label = r'' + manual_symbol + r' (' + manual_unit + r')'
+            else:
+                label = ''
 
     return label
 
@@ -330,14 +393,35 @@ def load_dict(filename):
         mydict = {}
         for rows in reader:
             if len(rows) > 0:
-                if rows[1] == '':
-                    mydict[rows[0]] = None
-                else:
-                    try:
-                        mydict[rows[0]] = eval(rows[1])  # if needs conversion
-                    except:
-                        mydict[rows[0]] = rows[1]  # if string
+                if len(rows) == 2:
+                    if rows[1] == '':
+                        mydict[rows[0]] = None
+                    else:
+                        try:
+                            mydict[rows[0]] = eval(rows[1])  # if needs conversion
+                            try:
+                                mydict[rows[0]] = eval(eval(rows[1]))  # if double string
+                            except:
+                                pass
+                        except:
+                            mydict[rows[0]] = rows[1]  # if string
+                elif len(rows) > 2:  # if line splitted because of comma, try to concatenate line
+                    val = ''
+                    for i in range(1,len(rows)-1):
+                        val += rows[i]+','
+                    val += rows[len(rows)-1]
 
+                    if val == '':
+                        mydict[rows[0]] = None
+                    else:
+                        try:
+                            mydict[rows[0]] = eval(val)  # if needs conversion
+                            try:
+                                mydict[rows[0]] = eval(eval(val))  # if double string
+                            except:
+                                pass
+                        except:
+                            mydict[rows[0]] = val  # if string
     return mydict
 
 
@@ -488,7 +572,7 @@ def pool_datasets(df_list, name_list):
         if df is None:
             continue
         df['dataset'] = name_list[i]
-        df_out = pd.concat([df_out, df])
+        df_out = pd.concat([df_out, df], ignore_index=True)
 
     return df_out
 
@@ -559,7 +643,8 @@ def get_info(data_dir):
                 info[k] = float(info[k])
 
     else:
-        raise Exception("ERROR: info.txt doesn't exist or is not at the main data folder")
+        print("WARNING: info.txt doesn't exist or is not at the main data folder")
+        return None
 
     mandatory_info = ['timescale', 'lengthscale']
     for mand_info in mandatory_info:
@@ -569,7 +654,7 @@ def get_info(data_dir):
 
 
 def get_data(data_dir, df=None, refresh=False, split_traj=False, set_origin_=False, image=None, reset_dim=['x', 'y'],
-             invert_axes=[]):
+             invert_axes=[], custom_var={}, trackmate_input=False):
     """
     Main function to import data and perform the initial processing (scaling and computing of time derivatives).
     It saves the database as a pickle object.
@@ -593,6 +678,10 @@ def get_data(data_dir, df=None, refresh=False, split_traj=False, set_origin_=Fal
     :type reset_dim: list
     :param invert_axes: list of dimensions to invert (change sign)
     :type invert_axes: list
+    :param custom_var: dict of custom variables: {'var_i':{'name':'var_name','unit':'var_unit'}}
+    :type custom_var: dict
+    :param trackmate_input: True if input file is a trackmate (v7) csv file
+    :type trackmate_input: bool
     :return: dict with dataframe and key info
     :rtype: dict
     """
@@ -615,9 +704,19 @@ def get_data(data_dir, df=None, refresh=False, split_traj=False, set_origin_=Fal
             data_file = osp.join(data_dir, 'positions.csv')
             sep = info["separator"] if "separator" in info.keys() else ','  # by default comma separated
             sep = '\t' if sep == 'tab' else sep
-            df = pd.read_csv(data_file, sep=sep)  # columns must be ['x','y','z','frame','track']
+            if osp.exists(data_file):
+                df = pd.read_csv(data_file, sep=sep)  # columns must be ['x','y','z','frame','track']
+            else: 
+                raise Exception("""data file does not exist, it must be named 'positions.csv' or 'positions.txt'""")
+
+        # trackmate specific case
+        if trackmate_input: 
+            df = df[['POSITION_X','POSITION_Y','POSITION_Z','FRAME','TRACK_ID']]
+            df.columns = ['x', 'y', 'z', 'frame', 'track']
+            df = df.loc[3:]
 
         # check data type
+        df = df.apply(pd.to_numeric, errors='ignore')
         dimensions = ['x', 'y', 'z'] if 'z' in df.columns else ['x', 'y']
         dim = len(dimensions)
         df['frame'] = df['frame'].astype(np.int)
@@ -646,11 +745,16 @@ def get_data(data_dir, df=None, refresh=False, split_traj=False, set_origin_=Fal
             df['z_rel'] = df['z_scaled'] - df['z_scaled'].mean()
 
         # update pickle
-        data = {'df': df, 'lengthscale': lengthscale, 'timescale': timescale, 'dim': dim, 'dimensions': dimensions}
+        data = {'df': df, 'lengthscale': lengthscale, 'timescale': timescale, 'dim': dim, 'dimensions': dimensions, 'custom_var':custom_var}
         pickle.dump(data, open(pickle_fn, "wb"))
     else:
         data = pickle.load(open(pickle_fn, "rb"))
 
+    # extra check that df is numerci (in case of old pickle)
+    df = data['df']
+    df = df.apply(pd.to_numeric, errors='ignore')
+    data['df'] = df
+
     return data
 
 
@@ -674,6 +778,8 @@ def get_traj(track_groups, track, min_frame=None, max_frame=None):
         group = group[group['frame'] >= min_frame]
     if max_frame is not None:
         group = group[group['frame'] <= max_frame]
+
+    group.sort_values(by='t', inplace=True)  # ensure sorted by t
     return group.reset_index(drop=True)
 
 
@@ -690,18 +796,26 @@ def filter_by_traj_len(df, min_traj_len=1, max_traj_len=None):
     :rtype: pandas.DataFrame
     """
 
+    # don't do anything if empty df
+    if df.shape[0] == 0:
+        return df
+
     if max_traj_len is None:  # assign the longest possible track
         max_traj_len = df['frame'].max() - df['frame'].min() + 1
     min_traj_len = 1 if min_traj_len is None else min_traj_len  # assign 1, if not given
 
-    tracks = df.groupby('track')
+    groups, group_values = get_unique_track_groups(df)
     df_list = []
-    for t in df['track'].unique():
-        track = tracks.get_group(t)
+    for group_value in group_values:
+        track = groups.get_group(group_value)
         if (track.shape[0] >= min_traj_len) & (track.shape[0] <= max_traj_len):
             df_list.append(track)
+    
+    if len(df_list) > 0:
+        out_df = pd.concat(df_list, ignore_index=True)
+    else: 
+        out_df = pd.DataFrame()  #return empty dataframe
 
-    out_df = pd.concat(df_list, ignore_index=True)
     return out_df
 
 
@@ -715,6 +829,11 @@ def filter_by_frame_subset(df, frame_subset=None):
     :return: filtered dataframe of trajectories
     :rtype: pandas.DataFrame
     """
+
+    # don't do anything if empty df
+    if df.shape[0] == 0:
+        return df
+
     if frame_subset is None:
         return df
     elif frame_subset[0] is None and frame_subset[1] is None:
@@ -728,8 +847,9 @@ def filter_by_frame_subset(df, frame_subset=None):
         df_ = df[((df['frame'] >= frame_subset[0]) & (df['frame'] <= frame_subset[1]))]
 
     if df_.shape[0] == 0:
-        print("WARNING: no data for this frame subset. Returning unfiltered database")
-        return df
+        print("WARNING: no data for this frame subset")
+        #print("WARNING: no data for this frame subset. Returning unfiltered database")
+        #return df
     return df_
 
 
@@ -747,6 +867,11 @@ def filter_by_region(df, xlim=None, ylim=None, zlim=None):
     :return: filtered dataframe of trajectories
     :rtype: pandas.DataFrame
     """
+
+    # don't do anything if empty df
+    if df.shape[0] == 0:
+        return df
+
     df_ = df.copy()
 
     dims = list('xyz')
@@ -763,65 +888,96 @@ def filter_by_region(df, xlim=None, ylim=None, zlim=None):
                 df_ = df_[((df_[dims[i]] >= lim_[0]) & (df_[dims[i]] <= lim_[1]))]
 
     if df_.shape[0] == 0:
-        print("WARNING: no data for this frame subset. Returning unfiltered database")
-        return df
+        print("WARNING: no data for this subset")
+        #print("WARNING: no data for this subset. Returning unfiltered database")
+        #return df
     return df_
 
 
-def get_coordinates(image, interactive=True, verbose=True):
+def get_coordinates(image, df=None, verbose=True):
     """
     Interactive selection of coordinates on an image by hand-drawing using a Napari viewer.
-    Selection supported: points and rectangle.
+    To be used with get_coordinates
     :param image: dict returned by get_image()
     :type image: dict
+    :param df: pd.dataframe of tracks
+    :type df: pandas.DataFrame
+    :param verbose: verbosity
+    :type verbose: bool
     :return: dict of list of selected shapes: {'points':[coordinates1,...],'rectangle':[coordinates1,...]}
     :rtype: dict
     """
 
-    image_fn = image['image_fn']
-    t_dim = image['t_dim']
-    z_dim = image['z_dim']
-
-    im = io.imread(image_fn)
-
-    selecting = True
-    while selecting:
-        # create a list to be modified in get_coord so it is not deleted when get_coord ends
-        shape_list = []
-        points_list = []
-        with napari.gui_qt():
-            viewer = napari.view_image(im)
-            if verbose:
-                print("Draw points or rectangles, then press ENTER and close the image viewer")
-
-            # retrieve coodinates on clicking Enter
-            @viewer.bind_key('Enter')
-            def get_coord(viewer):
-                for layer in viewer.layers:
-                    if type(layer) is napari.layers.shapes.shapes.Shapes:
-                        shape_list.append(layer)
-                    if type(layer) is napari.layers.points.points.Points:
-                        points_list.append(layer.data)
-
-        # inspect selected layers
-        rectangle_list = []
-        if len(shape_list) > 0:
-            for i, shape_type_ in enumerate(shape_list[0].shape_type):
-                if shape_type_ == 'rectangle':
-                    rectangle_list.append(shape_list[0].data[i])
-        points = np.array([])
-        if len(points_list) > 0:
-            points = points_list[0]
-
-        # interactive validation of selection 
-        if verbose:
-            print('You have selected {} point(s) and {} rectangle(s)'.format(points.shape[0], len(rectangle_list)))
-        if interactive:
-            finished = input('Is the selection correct? [y]/n: ')
-            if finished != 'n':
-                selecting = False
+    no_image = False
+    
+    if df is None and image is None: 
+        raise Exception("No data nor image")
+    
+    # if no image plot tracks in viewer
+    elif image is None: 
+        no_image = True
+    else: 
+        if image["image_fn"] is None:
+            no_image = True
         else: 
-            selecting = False
+            if not osp.exists(image['image_fn']):
+                print("Warning: {} does not exist".format(image['image_fn']))
+
+    if no_image: 
+        df = df.sort_values(by=['track', 'frame'])  # napari track layer requires data to be sorted by ID then frame
+        if 'z' in df.columns:
+            cols = ['frame', 'z', 'y', 'x'] 
+            t_dim = 0
+            z_dim = 1
+        else:
+            cols = ['frame', 'y', 'x']
+            t_dim = 0
+            z_dim = None
+        tracks = df[['track'] + cols].values
+
+    else: 
+        image_fn = image['image_fn']
+        t_dim = image['t_dim']
+        z_dim = image['z_dim']
+        im = io.imread(image_fn)
+
+    # create list to store layers' data from an open Napari viewer
+    shape_list = []
+    points_list = []
+
+    if no_image:
+        viewer = napari.Viewer()
+        viewer.add_tracks(tracks, name='trajectories')
+    else: 
+        viewer = napari.view_image(im)
+
+    # retrieve coodinates on clicking Enter
+    @viewer.bind_key('Enter')
+    def get_coord(viewer):
+        for layer in viewer.layers:
+            if type(layer) is napari.layers.shapes.shapes.Shapes:
+                shape_list.append(layer)
+            if type(layer) is napari.layers.points.points.Points:
+                points_list.append(layer.data)
+        viewer.close()
+
+    if verbose:
+        print("Draw points or rectangles, then press ENTER")
+
+    napari.run()
+
+    # inspect selected layers
+    rectangle_list = []
+    if len(shape_list) > 0:
+        for i, shape_type_ in enumerate(shape_list[0].shape_type):  # grab the first element to ignore if Enter is pressed several times
+            if shape_type_ == 'rectangle':
+                rectangle_list.append(shape_list[0].data[i])
+    points = np.array([])
+    if len(points_list) > 0:
+        points = points_list[0]  # grab the first element to ignore if Enter is pressed several times
+    
+    if verbose:
+        print('You have selected {} point(s) and {} rectangle(s)'.format(points.shape[0], len(rectangle_list)))
 
     # retreive coordinates
     coord_dict = {'points': [], 'rectangle': []}
@@ -883,6 +1039,10 @@ def filter_by_traj_id(df, track_list=None):
     :rtype: pandas.DataFrame
     """
 
+    # don't do anything if empty df
+    if df.shape[0] == 0:
+        return df
+
     if track_list is None:
         return df
     elif type(track_list) is float or type(track_list) is int:
@@ -914,7 +1074,7 @@ def select_traj_by_xyzt(df, xlim=None, ylim=None, zlim=None, frame_lim=None):
     :rtype: list
     """
     # filter by frame subset
-    if type(frame_lim) is list:
+    if type(frame_lim) is list or type(frame_lim) is tuple:
         df = filter_by_frame_subset(df, frame_subset=frame_lim)
     elif type(frame_lim) is int or type(frame_lim) is float:
         df = df[df['frame'] == frame_lim]
@@ -984,7 +1144,8 @@ def select_sub_data(df, filters=[]):
 
     # perform filtering
     df_list = []  # temp list of df 
-    for filt in filters:
+    subset_names = []
+    for i, filt in enumerate(filters):
         # filter
         df_ = filter_by_region(df, xlim=filt['xlim'], ylim=filt['ylim'], zlim=filt['zlim'])
         df_ = filter_by_frame_subset(df_, frame_subset=filt['frame_subset'])
@@ -995,17 +1156,32 @@ def select_sub_data(df, filters=[]):
                                             zlim=filt['track_ROI']['zlim'], frame_lim=filt['track_ROI']['frame_lim'])
             df_ = filter_by_traj_id(df_, track_list)
 
-        # subset name
-        df_['subset'] = filt['name']
-        df_list.append(df_)
+        if len(filters) > 1:  # populate subset_names only if there are several subset
+            subset_name = filt['name'] if filt['name'] != '' else 'subset_{}'.format(i)  # subset name
 
-    out_df = pd.concat(df_list, ignore_index=True)
+            # remove empty df_
+            if df_.shape[0] > 0:  
+                df_['subset'] = subset_name
+                subset_names.append(subset_name)
+            else: 
+                print("WARNING: subset {} has no data: Removing this subset... ".format(subset_name))
+        
+        if df_.shape[0] > 0:
+            df_list.append(df_)
 
-    return out_df
+    #check that subset name is not repeated
+    if len(subset_names) != len(set(subset_names)):
+        print("WARNING: some subset names are used several times")
+
+    if len(df_list) > 0:
+        out_df = pd.concat(df_list, ignore_index=True)
+        return out_df
+    else: 
+        raise Exception("WARNING: all datasets are empty. Aborting...")
 
 
 def get_background(image=None, frame=None, df=None, no_bkg=False, image_size=None, orig=None, axis_on=False,
-                   dpi=plot_param['dpi'], figsize=(5, 5)):
+                   dpi=plot_param['dpi'], figsize=(5, 5), figsize_factor=1):
     """Get image background or create white background if no_bkg. The image can be a time nd stack or a single image."""
     if orig is None:
         # orig = 'lower' if image_dir is None else 'upper' #trick to plot for the first time only inverting Yaxis: not very elegant...
@@ -1032,7 +1208,7 @@ def get_background(image=None, frame=None, df=None, no_bkg=False, image_size=Non
                 ymax = df['y'].max() + 0.05 * (df['y'].max() - df['y'].min())
                 figsize = ((xmax - xmin) / dpi, (ymax - ymin) / dpi)
 
-        fig = plt.figure(frameon=False, figsize=figsize)
+        fig = plt.figure(frameon=False, figsize=figsize, dpi=dpi*figsize_factor)
         ax = fig.add_axes([0, 0, 1, 1])
         ax.set_aspect('equal')
 
@@ -1054,7 +1230,7 @@ def get_background(image=None, frame=None, df=None, no_bkg=False, image_size=Non
         im = img_as_ubyte(im)  # 8bit conversion
         n = im.shape[0]
         m = im.shape[1]
-        fig = plt.figure(frameon=False, figsize=(m / dpi, n / dpi))
+        fig = plt.figure(frameon=False, figsize=(m / dpi, n / dpi), dpi=dpi*figsize_factor)
         ax = fig.add_axes([0, 0, 1, 1])
         ax.imshow(im, aspect='equal', origin=orig, cmap='gray', vmin=0, vmax=255)
         xmin, xmax, ymin, ymax = ax.axis()
@@ -1123,7 +1299,21 @@ def get_image(data_dir, filename=None, verbose=False):
 
         image_dict = {'image_fn': filename, 't_dim': t_dim, 'z_dim': z_dim, 'image_size': im.shape[-2:]}
     else:
-        image_dict = {'image_fn': None, 't_dim': None, 'z_dim': None, 'image_size': None}
+        # get image size
+        info = get_info(data_dir)
+
+        if info is None:
+            print("WARNING: no info.txt file so image size will be inferred")
+            image_size = None
+        else: 
+            if 'image_width' in info.keys() and 'image_height' in info.keys():
+                if info['image_width'] is not None and info['image_height'] is not None:
+                    image_size = (info['image_height'],info['image_width'])
+            else: 
+                image_size = None
+                print("WARNING: no image in directory, and image size is not defined in info.txt")
+
+        image_dict = {'image_fn': None, 't_dim': None, 'z_dim': None, 'image_size': image_size}
 
     return image_dict
 
@@ -1141,18 +1331,17 @@ def load_config(data_dir, verbose=False):
                 if f.endswith('.csv'):
                     out_dict[f[:-4]] = load_dict(osp.join(config_dir, f))
         else:
-            if verbose:
-                print("WARNING: config is not a directory. Config not loaded.")
+            print("WARNING: config is not a directory. Config not loaded.")
     else:
-        if verbose:
-            print("WARNING: no config directory. Config not loaded.")
+        print("WARNING: no config directory. Config not loaded.")
 
     return out_dict
 
 
-def make_map_config(data_dir=None, export_config=True):
+def make_map_config(data_dir=None, export_config=True, empty_run=True):
     """
     Generate a list of dictionaries containing the parameters used to run map_analysis.
+    empty_run allows not to use the default parameters to analyze and to skip the analysis
     """
 
     # config to create grid using prepare.make_grid()
@@ -1175,6 +1364,12 @@ def make_map_config(data_dir=None, export_config=True):
 
     # config of scalar field to plot. Each key is a parameter to plot and it stores a dict containing the plotting config for this specific parameter
     scalar_fields = {'vx': {'vlim': None,  # value limits to display on the color map
+                            'cmap': "plasma"  # color map
+                            },
+                    'vy': {'vlim': None,  # value limits to display on the color map
+                            'cmap': "plasma"  # color map
+                            },
+                    'v': {'vlim': None,  # value limits to display on the color map
                             'cmap': "plasma"  # color map
                             }
                      }
@@ -1193,6 +1388,12 @@ def make_map_config(data_dir=None, export_config=True):
                               }
                    }
 
+    if empty_run: 
+        # erase analysis config
+        scalar_fields = {}
+        vector_fields = {}
+        vector_mean = {}
+
     # package all in a dict
     config = {'grid_param': grid_param,
               'map_param': map_param,
@@ -1221,7 +1422,7 @@ def make_traj_config(data_dir=None, export_config=True):
     """
 
     traj_config_ = {'run': True,  # run plot_traj
-                    'color_code': 'z',  # color code: 'z', 'ROI', 'random', 'none'
+                    'color_code': 'z',  # color code: 'z', 'group', 'random', 'none'
                     'cmap': 'plasma',  # colormap to be used if color_code is 'z'
                     'cmap_lim': None,
                     # pass custom colormap limits (useful for getting coherent boundaries for all frames)
@@ -1234,7 +1435,7 @@ def make_traj_config(data_dir=None, export_config=True):
                     'plot3D': False,  # plot in 3D  !! Not supportes anymore !!
                     'elevation': None,  # 3D paramater !! Not supportes anymore !!
                     'angle': None,  # 3D paramater !! Not supportes anymore !!
-                    'subset_order': None,  # if color code is ROI, order of ROI in color cycle
+                    'subset_order': None,  # if color_code is group, order of group in color cycle
                     }
 
     MSD_config = {'run': True,  # run plot_all_MSD
@@ -1250,28 +1451,42 @@ def make_traj_config(data_dir=None, export_config=True):
                   }
 
     scatter_config = {'run': True,  # run param_vs_param
-                      'couple_list': [['x', 'v'], ['y', 'v']],
-                      # list of [x,y] couples of variables to be plotted in scatter
-                      'mean_couple_list': [['x', 'v'], ['y', 'v']]
-                      # list of [x,y] couples of variables averaged along the whole track to be plotted in scatter
+                      'couple_list': [['x', 'v'], ['y', 'v']],  # list of [x,y] couples of variables to be plotted in scatter
+                      'mean_couple_list': [['x', 'v'], ['y', 'v']],  # list of [x,y] couples of variables averaged along the whole track to be plotted in scatter
+                      'x_bin_num': None,  #  number of evenly spaced bins along x axis
+                      'ci': None,  # show confidence interval [0,100]
+                      'fit_reg': False,  # show regression fit
+                      'scatter': True,  # show scatter 
                       }
 
     hist_config = {'run': True,  # run plot_param_hist
                    'var_list': ['v'],  # list of variables to be plotted in histogram
-                   'mean_var_list': ['v']
-                   # list of variables averaged along the whole track to be plotted in histogram
+                   'mean_var_list': ['v']  # list of variables averaged along the whole track to be plotted in histogram
                    }
 
+    boxplot_config = {'run': True,  # run plot_param_hist
+                       'var_list': ['v'],  # list of variables to be plotted 
+                       'mean_var_list': ['v'],  # list of variables averaged along the whole track to be plotted
+                       'save_stat': True ,  # run statistical test (scipy.stats.ttest_ind(equal_var=False))
+                       'boxplot': True,  # show boxplot for variables in var_list
+                       'swarmplot': False,  # show swarmplot for variables in var_list
+                       'mean_boxplot': True,  # show boxplot for variables in mean_var_list
+                       'mean_swarmplot': True,  # show swarmplot for variables in mean_var_list
+                       'mean_save_stat': True ,  # run statistical test (scipy.stats.ttest_ind(equal_var=False)) for wholetrack parameters
+                       }
+
     total_traj_config = {'run': True,  # run plot_centered_traj
-                         'hide_labels': False,  # hide trajectory ID
+                         'hide_labels': True,  # hide trajectory ID
                          'label_size': 5,  # label size in points if hide_labels is False
-                         'dont_center': False,  # to keep initial position at its true position
+                         'center_origin': True,  # to center origin of all trajectories
                          'set_axis_lim': None,  # custom axis limites: [xmin,xmax,ymin,ymax]
                          'equal_axis': True,  # set x and y axes' scaling equal
-                         'color_code': 'random',  # color code: 'z', 'ROI', 'random', 'none'
+                         'color_code': 'random',  # color code: 'z', 'group', 'random', 'none'
                          'cmap': 'plasma',  # colormap to be used if color_code is 'z'
                          'cmap_lim': None,  # pass custom colormap limits
-                         'subset_order': None,  # if color code is ROI, order of ROI in color cycle
+                         'subset_order': None,  # if color_code is group, order of group in color cycle
+                         'transparency': 1,  #plot transparency
+                         'show_legend': True  # show legend
                          }
 
     # config of voronoi plot
@@ -1284,7 +1499,8 @@ def make_traj_config(data_dir=None, export_config=True):
                 'area_threshold': 3,  # exclude areas above this multiple of area median
                 'no_bkg': False,  # don't show background image if an image is passed
                 'size_factor': 1.,  # to multiply the default size of lines !! Not implemented yet !!
-                'show_axis': False,  # to show the plot axes (by default just image) 
+                'show_axis': False,  # to show the plot axes (by default just image)
+                'line_width': 1.,  # diagram line width
                }
 
     # package all in a dict
@@ -1294,6 +1510,7 @@ def make_traj_config(data_dir=None, export_config=True):
               'hist_config': hist_config,
               'total_traj_config': total_traj_config,
               'voronoi_config': voronoi,
+              'boxplot_config': boxplot_config,
               }
 
     if export_config:
@@ -1315,8 +1532,8 @@ def make_data_config(data_dir=None, export_config=True):
     Initialize parameters used by get_data
     :param data_dir: path to data directory
     :type data_dir: str or None
-    :param export_to_config: to export to config file
-    :type export_to_config: bool
+    :param export_config: to export to config file
+    :type export_config: bool
     :return: filters used by select_sub_data()
     :rtype: dict
     """
@@ -1325,6 +1542,7 @@ def make_data_config(data_dir=None, export_config=True):
               'set_origin_': False,  # to reset origin of coordinates with dict with dimensions as key and coordinates as values
               'reset_dim': ['x', 'y'],  # list of dimensions to reset, as set_origin_ contains coordinates along all dimensions
               'invert_axes': [],  # list of dimensions to invert (change sign)
+              'trackmate_input': False,  # if the input file is a trackmate (v7) csv file
              }
 
     if export_config: 
@@ -1339,13 +1557,13 @@ def make_data_config(data_dir=None, export_config=True):
     return config
 
 
-def init_filters(data_dir=None, export_to_config=False):
+def init_filters(data_dir=None, export_config=False):
     """
     Initialize database filters
     :param data_dir: path to data directory
     :type data_dir: str or None
-    :param export_to_config: to export to config file
-    :type export_to_config: bool
+    :param export_config: to export to config file
+    :type export_config: bool
     :return: filters used by select_sub_data()
     :rtype: dict
     """
@@ -1366,7 +1584,7 @@ def init_filters(data_dir=None, export_to_config=False):
                 'subset_order': None,  # to give a custom order from plotting subset together
                 }
 
-    if export_to_config: 
+    if export_config: 
         if data_dir is None:
             raise Exception("ERROR: no data_dir given")
         else:
@@ -1384,7 +1602,7 @@ def make_all_config(data_dir=None, export_config=True):
     """
 
     data_config = make_data_config(data_dir=data_dir, export_config=export_config)
-    filters = init_filters(data_dir=data_dir, export_to_config=export_config)
+    filters = init_filters(data_dir=data_dir, export_config=export_config)
     traj_config = make_traj_config(data_dir=data_dir, export_config=export_config)
     map_config = make_map_config(data_dir=data_dir, export_config=export_config)
     plot_config = tpl.make_plot_config(data_dir=data_dir, export_config=export_config)
@@ -1396,3 +1614,26 @@ def make_all_config(data_dir=None, export_config=True):
                     "plot_config": plot_config,
                     }
     return all_config
+
+
+def get_unique_track_groups(df):
+    """
+    Split a df in track groups making sure they are unique tracks (splitting by subsets too if exist)
+    Return pandas groups and the values to get them
+    """
+    if 'subset' in df.columns: 
+        groups = df.groupby(by=['subset','track'])
+        df_ = df.drop_duplicates(subset=['subset','track']) # get only unique tracks
+        group_values_arr = df_[['subset','track']].values
+        
+        # transform group_values_arr to a list of tuple
+        group_values = []
+        for i in range(group_values_arr.shape[0]):
+            group_values.append((group_values_arr[i,0],group_values_arr[i,1]))
+
+    else: 
+        groups = df.groupby(by='track')
+        group_values = df['track'].unique()
+
+    return groups, group_values
+
